import streamlit as st
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder
from sklearn.metrics import classification_report, confusion_matrix, ConfusionMatrixDisplay

# Preprocess the dataset
def preprocess_data(df):
    """Preprocess the combined DataFrame for analysis and modeling."""
    # Handle missing values in key columns
    df["release_year"] = df["release_year"].fillna(df["release_year"].median())
    df["rating"] = df["rating"].fillna("Not Rated")
    df["date_added"] = pd.to_datetime(df["date_added"], errors="coerce").fillna(pd.Timestamp("2020-01-01"))
    df["duration"] = df["duration"].fillna("0 min")
    df["director"] = df["director"].fillna("Unknown")

    # Clean the 'duration' column to extract numeric values
    def preprocess_duration(row):
        if "min" in row:
            return int(row.replace(" min", ""))
        elif "Season" in row:
            return int(row.replace(" Season", "").replace("s", ""))
        return None

    df["duration_numeric"] = df["duration"].apply(preprocess_duration)

    # Drop rows where duration could not be parsed
    df = df[df["duration_numeric"].notnull()]

    # Extract primary genre from 'listed_in' and encode it
    df["primary_genre"] = df["listed_in"].str.split(',').str[0].str.strip()
    df["primary_genre"] = df["primary_genre"].fillna("Other")
    return df

# Main page for adding and applying the model
def add_and_apply_model_page():
    st.title("🎥 Netflix Dataset: Predict Genre with Logistic Regression")

    # Step 1: Load the Combined Dataset
    st.header("📂 1. Load Combined Dataset")
    try:
        combined_df = pd.read_csv("netflix_combined_dataset.csv")  # Replace with your combined dataset path
        st.success("✅ Combined dataset loaded successfully!")
        st.dataframe(combined_df.head())
    except Exception as e:
        st.error(f"❌ Error loading combined dataset: {e}")
        return

    # Step 2: Preprocess the Combined Dataset
    st.header("🔄 2. Preprocess Combined Dataset")
    combined_df = preprocess_data(combined_df)

    # Use only the first 1000 rows for the prediction
    combined_df = combined_df.head(1000)
    st.success("✅ Preprocessing completed!")
    st.dataframe(combined_df.head())

    # Step 3: Encode Director and Genre
    st.header("🔢 3. Encode Features and Target")
    director_encoder = LabelEncoder()
    genre_encoder = LabelEncoder()

    combined_df["director_encoded"] = director_encoder.fit_transform(combined_df["director"])
    combined_df["genre_encoded"] = genre_encoder.fit_transform(combined_df["primary_genre"])

    # Filter out genres with fewer than 2 samples
    genre_counts = combined_df["genre_encoded"].value_counts()
    valid_genres = genre_counts[genre_counts >= 2].index
    combined_df = combined_df[combined_df["genre_encoded"].isin(valid_genres)]

    # Step 4: Feature and Target Selection
    st.header("🎯 4. Feature and Target Selection")
    features = ["director_encoded", "duration_numeric", "release_year"]  # Features for prediction
    target = "genre_encoded"  # Target variable
    X = combined_df[features]
    y = combined_df[target]
    st.write("### Features (X)")
    st.dataframe(X.head())
    st.write("### Target (y)")
    st.write(y.head())

    # Step 5: Train-Test Split
    st.header("🔀 5. Train-Test Split")
    test_size = st.slider("Select Test Size Percentage", min_value=10, max_value=50, value=20, step=5)
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size / 100, random_state=42, stratify=y)
    st.success(f"✅ Data split: {100 - test_size}% training and {test_size}% testing.")

    # Step 6: Train Logistic Regression Model
    st.header("🤖 6. Train Logistic Regression Model")
    model = LogisticRegression(max_iter=1000, random_state=42)
    model.fit(X_train, y_train)
    st.success("✅ Logistic Regression Model trained successfully!")

    # Step 7: Model Predictions
    st.header("🔍 7. Model Predictions")
    y_pred = model.predict(X_test)
    comparison_df = pd.DataFrame({
        "Actual": genre_encoder.inverse_transform(y_test),
        "Predicted": genre_encoder.inverse_transform(y_pred)
    })
    st.dataframe(comparison_df.head())

    # Step 8: Visualization of Predictions
    st.header("📈 8. Visualization: Actual vs Predicted")
    fig, ax = plt.subplots(figsize=(10, 6))
    sns.countplot(x="Actual", data=comparison_df, color="blue", alpha=0.6, label="Actual")
    sns.countplot(x="Predicted", data=comparison_df, color="orange", alpha=0.6, label="Predicted")
    ax.set_xticklabels(ax.get_xticklabels(), rotation=45, ha="right")
    plt.title("Actual vs Predicted Genre Counts")
    plt.xlabel("Genre")
    plt.ylabel("Count")
    plt.legend()
    st.pyplot(fig)

    # Step 9: Custom Input for Prediction
    st.header("🛠️ 9. Custom Prediction")

    # Add a placeholder for director selection
    directors = ["Select a Director"] + sorted(combined_df["director"].unique())
    selected_director = st.selectbox("Select a Director", directors)

    # Add sliders for duration and release year with no default values
    duration_choice = st.slider(
        "Enter Duration (in minutes)",
        min_value=int(combined_df["duration_numeric"].min()),
        max_value=int(combined_df["duration_numeric"].max()),
    )
    release_year_choice = st.slider(
        "Enter Release Year",
        min_value=int(combined_df["release_year"].min()),
        max_value=int(combined_df["release_year"].max()),
    )

    # Default message for predicted genre
    if selected_director == "Select a Director":
        st.header("**Predicted Genre:** ❓ 🤷 ❓")
    else:
        # Encode the input
        director_encoded = director_encoder.transform([selected_director])[0]
        custom_input = [[director_encoded, duration_choice, release_year_choice]]

        # Predict the genre
        custom_prediction = genre_encoder.inverse_transform(model.predict(custom_input))[0]
        st.header(f"**Predicted Genre:**  ✨ {custom_prediction} ✨")


    # Step 10: Save Model and Data to Session State
    st.session_state.model = model
    st.session_state.combined_data = combined_df
    st.success("✅ Model applied successfully!")

add_and_apply_model_page()
