import streamlit as st
import pandas as pd
import numpy as np

from sklearn.preprocessing import StandardScaler, LabelEncoder
from sklearn.model_selection import train_test_split, cross_val_score
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import classification_report, confusion_matrix

def algorithm_selection_page():
    st.title("🧠 Algorithm Selection for Netflix Genre Prediction")

    # ---------------------------
    # 1. Load and Preprocess Data
    # ---------------------------
    st.markdown("## 🔄 Data Loading and Preprocessing")
    file_path = "netflix_titles.csv"  # Update with the correct file path
    df = pd.read_csv(file_path)

    # Handle missing values in critical columns
    df["listed_in"] = df["listed_in"].fillna("Unknown")  # Fill missing genres
    df["release_year"] = df["release_year"].fillna(df["release_year"].median())
    df["duration"] = df["duration"].fillna("0 min")  # Default to "0 min" for missing durations

    # Clean the 'duration' column (convert "90 min" -> 90)
    def clean_duration(value):
        if "min" in str(value):
            return int(value.replace(" min", "").strip())
        elif "Season" in str(value):  # Handle TV Shows
            return 0
        return 0  # Default for unrecognized values

    df["duration"] = df["duration"].apply(clean_duration)

    # Extract primary genre from 'listed_in' and group rare genres as "Other"
    df['primary_genre'] = df['listed_in'].str.split(',').str[0].str.strip()
    df['primary_genre'] = df['primary_genre'].fillna("Other")
    common_genres = df['primary_genre'].value_counts()[df['primary_genre'].value_counts() >= 5].index
    df['primary_genre'] = df['primary_genre'].apply(lambda x: x if x in common_genres else 'Other')

    # Encode the target variable (genre)
    genre_encoder = LabelEncoder()
    df["genre_encoded"] = genre_encoder.fit_transform(df["primary_genre"])

    # Scale numeric features: duration and release_year
    scaler = StandardScaler()
    df["duration_scaled"] = scaler.fit_transform(df[["duration"]])
    df["release_year_scaled"] = scaler.fit_transform(df[["release_year"]])

    # ---------------------------
    # 2. Feature and Target Setup
    # ---------------------------
    st.markdown("## 🛠️ Feature Selection")

    # Select features for training and the target column
    selected_features = ["duration_scaled", "release_year_scaled"]
    target_col = "genre_encoded"

    X = df[selected_features]
    y = df[target_col]

    # Display selected features and target variable
    st.markdown("### Selected Features (X)")
    st.write(X.head())
    st.markdown("### Target Variable (Genre Encoded)")
    st.write(y.head())

    # ---------------------------
    # 3. Train-Test Split
    # ---------------------------
    st.markdown("## 🔀 Train/Test Split")
    
    # Let the user set the test set size
    test_size = st.slider("Test Set Size (%)", min_value=10, max_value=50, value=20, step=5)
    X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=test_size/100, random_state=42)

    st.write(f"Training set size: {len(X_train)}")
    st.write(f"Validation set size: {len(X_val)}")

    # ---------------------------
    # 4. Model Training and Scoring
    # ---------------------------
    st.markdown("## 🤖 Model Training and Comparison")

    # ---- Model A: Logistic Regression ----
    st.markdown("### Logistic Regression")
    logreg = LogisticRegression(max_iter=1000, random_state=42)
    logreg.fit(X_train, y_train)
    y_pred_logreg = logreg.predict(X_val)
    logreg_accuracy = logreg.score(X_val, y_val)

    st.write(f"Accuracy on Validation Set: **{logreg_accuracy:.4f}**")
    cv_scores_log = cross_val_score(logreg, X, y, cv=5, scoring='accuracy')
    st.write("Cross-Validation Scores:")
    st.write(cv_scores_log)

    # ---- Model B: Random Forest ----
    st.markdown("### Random Forest")
    rf_model = RandomForestClassifier(n_estimators=100, random_state=42)
    rf_model.fit(X_train, y_train)
    y_pred_rf = rf_model.predict(X_val)
    rf_accuracy = rf_model.score(X_val, y_val)

    st.write(f"Accuracy on Validation Set: **{rf_accuracy:.4f}**")
    cv_scores_rf = cross_val_score(rf_model, X, y, cv=5, scoring='accuracy')
    st.write("Cross-Validation Scores:")
    st.write(cv_scores_rf)

    # ---------------------------
    # 5. Detailed Model Comparison
    # ---------------------------
    st.markdown("## 📊 Model Comparison")

    # Compare models in a small DataFrame
    comparison_df = pd.DataFrame({
        "Model": ["Logistic Regression", "Random Forest"],
        "Validation Accuracy": [logreg_accuracy, rf_accuracy],
        "CV Accuracy (mean)": [cv_scores_log.mean(), cv_scores_rf.mean()]
    })
    st.markdown("### Comparison Table")
    st.write(comparison_df)

    # Highlight the best model
    best_idx = comparison_df["Validation Accuracy"].idxmax()
    best_model_info = comparison_df.iloc[best_idx]
    st.markdown("### Best Model")
    st.write(best_model_info)

    # Save results to session state for future use
    st.session_state.comparison_df = comparison_df
    st.session_state.best_model_info = best_model_info

    st.success("✅ Algorithm selection and comparison completed!")
    st.success("✅ Algorithm Selected is Logistic Regression!")

algorithm_selection_page()
