import streamlit as st
import pandas as pd
import random
import numpy as np
from sklearn.preprocessing import StandardScaler, LabelEncoder

# Constants for random data generation
YEAR_MIN = 1920
YEAR_MAX = 2023
DURATION_MIN = 1
DURATION_MAX = 300  # Maximum duration in minutes or episodes

def load_data(file_path: str) -> pd.DataFrame:
    """Load the Netflix dataset from a CSV file."""
    try:
        return pd.read_csv(file_path)
    except FileNotFoundError:
        # Show error message if the file isn't found
        st.error(f"❌ File not found at path: {file_path}. Please check the file path.")
        return pd.DataFrame()

def preprocess_data(df: pd.DataFrame):
    """Preprocess the dataset for augmentation."""
    # Drop rows with missing essential data
    df = df.dropna(subset=['duration', 'director', 'release_year', 'listed_in'])

    # Extract numeric duration values (e.g., "120 min" → 120)
    def clean_duration(value):
        if "min" in str(value):
            return int(value.replace(" min", "").strip())
        return 0  # Default to 0 if unrecognized

    df['duration_numeric'] = df['duration'].apply(clean_duration)
    df = df[df['duration_numeric'] >= 0]  # Keep valid durations

    # Simplify genres by focusing on the primary genre and grouping rare ones as 'Other'
    df['primary_genre'] = df['listed_in'].str.split(',').str[0].str.strip()
    common_genres = df['primary_genre'].value_counts()[df['primary_genre'].value_counts() >= 5].index.tolist()
    df['primary_genre'] = df['primary_genre'].apply(lambda g: g if g in common_genres else 'Other')

    # Encode genres and count directors for further analysis
    genre_encoder = LabelEncoder()
    df['genre_encoded'] = genre_encoder.fit_transform(df['primary_genre'])
    director_counts = df['director'].value_counts()
    df['director_count'] = df['director'].map(director_counts)

    # Scale numeric features for consistency
    duration_scaler = StandardScaler()
    release_year_scaler = StandardScaler()
    df['duration_scaled'] = duration_scaler.fit_transform(df[['duration_numeric']])
    df['release_year_scaled'] = release_year_scaler.fit_transform(df[['release_year']])

    return df, genre_encoder, duration_scaler, release_year_scaler, director_counts

def generate_fake_row(genre_encoder, common_genres, director_counts):
    """Generate a single synthetic data row."""
    # Randomly select a genre and encode it
    primary_genre = random.choice(common_genres + ['Other'])
    genre_encoded = genre_encoder.transform([primary_genre])[0]

    # 70% chance to use an existing director, 30% to create a new one
    if random.random() < 0.7 and not director_counts.empty:
        director = random.choice(director_counts.index.tolist())
        director_count = director_counts.get(director, 1) + 1
    else:
        director = f"Director {random.randint(1001, 2000)}"
        director_count = 1

    # Generate type, duration, and other fields
    type_choice = random.choice(['Movie', 'TV Show'])
    duration_numeric = random.randint(DURATION_MIN, DURATION_MAX)
    duration = f"{duration_numeric} min" if type_choice == "Movie" else f"{duration_numeric} episodes"

    return {
        'type': type_choice,
        'title': f"Random Title {random.randint(1, 1000)}",
        'director': director,
        'duration': duration,
        'release_year': random.randint(YEAR_MIN, YEAR_MAX),
        'primary_genre': primary_genre,
        'genre_encoded': genre_encoded,
        'director_count': director_count,
        'duration_numeric': duration_numeric,
    }

def augmentation_page():
    st.title("🛠️ Netflix Dataset Augmentation")  # Main page title
    st.markdown(
        """
        <style>
        .header {
            background-color: #4caf50;
            color: white;
            padding: 15px;
            text-align: center;
            border-radius: 10px;
        }
        </style>
        <div class="header">
            Enhance your Netflix dataset with synthetic data for improved analysis!
        </div>
        """,
        unsafe_allow_html=True,
    )

    # Load Netflix dataset
    file_path = "netflix_titles.csv"
    df = load_data(file_path)

    # Exit if the dataset is empty
    if df.empty:
        st.warning("⚠️ Dataset is empty. Please check the file path and try again.")
        return

    # Preprocess the dataset to clean and encode necessary features
    df, genre_encoder, duration_scaler, release_year_scaler, director_counts = preprocess_data(df)

    # Display the cleaned and preprocessed dataset
    st.markdown("## 🔍 Preprocessed Original Dataset")
    st.write(df.head())

    # User input for synthetic data generation
    st.markdown("## 🔧 Generate Synthetic Data")
    num_rows = st.slider("Number of rows to generate:", 10, 1000, step=10, value=100)

    # Generate synthetic rows with a progress bar
    rows = []
    progress_bar = st.progress(0)
    for i in range(num_rows):
        rows.append(generate_fake_row(genre_encoder, df['primary_genre'].unique().tolist(), director_counts))
        progress_bar.progress((i + 1) / num_rows)
    progress_bar.empty()  # Clear the progress bar

    # Create a synthetic dataset
    fake_df = pd.DataFrame(rows)

    # Scale synthetic data columns
    fake_df['duration_scaled'] = duration_scaler.transform(fake_df[['duration_numeric']])
    fake_df['release_year_scaled'] = release_year_scaler.transform(fake_df[['release_year']])

    # Display comparison: original vs synthetic data
    st.markdown("## 📊 Dataset Comparison")
    cols = st.columns(2)
    with cols[0]:
        st.markdown("### Original Data")
        st.dataframe(df.head())
    with cols[1]:
        st.markdown("### Synthetic Data")
        st.dataframe(fake_df.head())

    # Display summary metrics for both datasets
    st.markdown("## 📈 Dataset Metrics")
    metrics_cols = st.columns(2)
    with metrics_cols[0]:
        st.markdown("### Original Data Metrics")
        st.write(df.describe(include="all"))
    with metrics_cols[1]:
        st.markdown("### Synthetic Data Metrics")
        st.write(fake_df.describe(include="all"))

    # Option to combine original and synthetic datasets
    st.markdown("## 📦 Combine Datasets")
    if st.button("Combine and Download"):
        combined_df = pd.concat([df, fake_df], ignore_index=True)
        st.write(combined_df.head())
        st.write(f"**Combined Dataset Size:** {combined_df.shape[0]} rows")
        csv = combined_df.to_csv(index=False).encode('utf-8')
        st.download_button(
            "Download Combined Dataset as CSV",
            csv,
            "netflix_combined_dataset.csv",
            "text/csv",
        )

augmentation_page()
