import streamlit as st
import matplotlib.pyplot as plt
import pandas as pd

def data_metrics_page():
    st.title("📊 Netflix Dataset Metrics and Visualization")  # Main title with emoji

    # Step 1: Load the Netflix dataset
    file_path = "netflix_titles.csv"
    df = pd.read_csv(file_path)
    st.session_state.df = df  # Save the dataset in session state for reuse

    # Step 2: Dataset Overview
    st.markdown("## 🔍 Dataset Overview")

    st.markdown("### Head of the Data")
    st.write(df.head())  # Display the first few rows of the dataset

    st.markdown("### Summary Statistics")
    st.write(df.describe(include='all'))  # Include all columns, even non-numeric

    st.markdown("### Data Types")
    st.write(df.dtypes)  # Show data types of all columns

    st.markdown("### Null Values")
    st.write(df.isnull().sum())  # Count and display missing values for each column


    # Step 3: Visualizations
    st.markdown("## 📈 Visualizations")

    # Visualization 1: Content Type Distribution
    if "type" in df.columns:
        st.markdown("### Distribution of Content Types")
        type_counts = df["type"].value_counts()  # Count occurrences of each content type
        fig, ax = plt.subplots()
        ax.bar(type_counts.index, type_counts.values, color='skyblue')
        ax.set_xlabel("Type")
        ax.set_ylabel("Count")
        ax.set_title("Distribution of Content Types")
        st.pyplot(fig)  # Display the bar chart

    # Visualization 2: Release Year Distribution
    if "release_year" in df.columns:
        st.markdown("### Number of Titles Released Over the Years")
        year_counts = df["release_year"].value_counts().sort_index()  # Count by release year
        fig, ax = plt.subplots()
        ax.plot(year_counts.index, year_counts.values, marker='o', color='green')
        ax.set_xlabel("Year")
        ax.set_ylabel("Count")
        ax.set_title("Number of Titles Released Over the Years")
        st.pyplot(fig)  # Display the line plot

    # Visualization 3: Duration Histogram
    if "duration" in df.columns:
        st.markdown("### Histogram of Durations (Movies/TV Shows)")
        # Extract numeric duration values (e.g., '90 min' → 90)
        df["duration_numeric"] = pd.to_numeric(df["duration"].str.extract(r'(\d+)')[0], errors='coerce')
        fig, ax = plt.subplots()
        ax.hist(df["duration_numeric"].dropna(), bins=20, edgecolor='black', color='orange')
        ax.set_xlabel("Duration")
        ax.set_ylabel("Count")
        ax.set_title("Histogram of Durations")
        st.pyplot(fig)  # Display the histogram

    # Step 4: Success Message
    st.success("✅ Data metrics and visualizations are complete!")

data_metrics_page()
