import streamlit as st
import pandas as pd
from sklearn.preprocessing import LabelEncoder

def feature_engineering_page():
    st.title("📊 Netflix Dataset Feature Engineering")

    # Step 1: Check for Preprocessed Data
    if 'df' not in st.session_state:
        st.error("⚠️ Preprocessed data not found in session state. Please complete the Preprocessing step first.")
        return

    # Work on a copy of the preprocessed dataset
    df = st.session_state.df.copy()

    # Step 2: Display Preprocessed Dataset
    st.markdown("## 🔍 Preprocessed Dataset")
    st.write(df.head())

    # Step 3: Check for Required Columns
    required_columns = ['duration', 'director', 'release_year', 'listed_in']
    missing_columns = [col for col in required_columns if col not in df.columns]

    if missing_columns:
        st.error(f"❌ The following required columns are missing: {missing_columns}")
        st.info("Ensure the Preprocessing step has been completed successfully.")
        return

    # Step 4: Feature Selection for Modeling
    st.markdown("## 🛠️ Selecting Features for Modeling")

    # Define feature columns and target variable
    feature_columns = ["duration", "director", "release_year"]
    df['primary_genre'] = df['listed_in'].str.split(',').str[0].str.strip()  # Extract primary genre
    target_column = "primary_genre"

    # Display selected features and target variable
    X = df[feature_columns]
    y = df[target_column]
    st.markdown("### Features (X)")
    st.write(X.head())
    st.markdown("### Target Variable (y - Primary Genre)")
    st.write(y.head())

    # Step 5: Encode Additional Categorical Variables
    st.markdown("## 🔄 Encoding Categorical Variables")

    # Identify categorical columns to encode (excluding already processed ones)
    categorical_cols = df.select_dtypes(include=['object']).columns.tolist()
    excluded_cols = [
        'show_id', 'title', 'cast', 'country', 'date_added',
        'rating', 'duration', 'listed_in', 'description', 'primary_genre'
    ]
    categorical_cols = [col for col in categorical_cols if col not in excluded_cols]

    if categorical_cols:
        label_encoders = {}
        for col in categorical_cols:
            le = LabelEncoder()
            df[col] = le.fit_transform(df[col])  # Apply Label Encoding
            label_encoders[col] = le
            st.markdown(f"### Encoded Column: `{col}`")
            st.write(df[[col]].head())  # Display encoded column
        st.session_state.label_encoders = label_encoders  # Save encoders to session state
    else:
        st.info("No additional categorical variables found for encoding.")

    # Step 6: Save Features and Target for Further Modeling
    st.session_state.features = X
    st.session_state.target = y
    st.success("✅ Feature engineering is complete! Features and target are ready for modeling.")

feature_engineering_page()
