import streamlit as st
import pandas as pd
from sklearn.linear_model import LinearRegression, LogisticRegression
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import accuracy_score

def model_application_page():
    # Main title with emojis
    st.title("🎥 Netflix Dataset Model Application")
    st.markdown(
        """
        <style>
        .main-header {
            background-color: #1DB954;
            color: white;
            padding: 15px;
            text-align: center;
            font-size: 24px;
            border-radius: 10px;
        }
        </style>
        <div class="main-header">
            Unlock the power of Netflix data with personalized recommendations, insights, and trends!
        </div>
        """,
        unsafe_allow_html=True,
    )

    # Load the Netflix dataset
    file_path = "netflix_titles.csv"
    df = pd.read_csv(file_path)

    st.markdown("---")  # Section divider

    # 1. Personalized Playlist
    st.header("📋 1. Personalized Netflix Recommendations")

    # Preprocessing
    valid_ratings = [
        "TV-MA", "PG-13", "R", "PG", "G", "NC-17", "TV-Y", "TV-Y7",
        "TV-G", "TV-PG", "TV-14", "NR", "UR"
    ]
    df["rating"] = df["rating"].apply(lambda x: x if x in valid_ratings else "NR")
    df["release_year"] = df["release_year"].fillna(df["release_year"].median())
    df["rating"] = df["rating"].fillna("Not Rated")
    df["genres"] = df["listed_in"].str.split(", ")

    # User Inputs
    st.markdown("### 🎯 Input Your Preferences")
    available_genres = sorted(set(df["genres"].explode().dropna()))
    preferred_genres = st.multiselect("🎥 Select Genres:", available_genres, default=[])
    available_ratings = sorted(df["rating"].unique())
    preferred_ratings = st.multiselect("⭐ Select Ratings:", available_ratings, default=[])

    # Filter Data by Genre
    filtered_df = df[df["genres"].apply(lambda genres: any(genre in genres for genre in preferred_genres))]
    filtered_df = filtered_df[filtered_df["rating"].isin(preferred_ratings)]
    filtered_df = filtered_df.sort_values(by=["release_year"], ascending=False)

    # Recommendations
    recommendations = filtered_df.head(20)
    st.markdown("### 📃 Top 20 Recommendations for You")
    if recommendations.empty:
        st.warning("No recommendations found for the selected criteria. Please adjust your preferences.")
    else:
        st.write(recommendations[["title", "type", "release_year", "rating", "listed_in"]])

    st.markdown("---")  # Section divider

    # 2. Country-Wise Popular Content
    st.header("🌎 2. Country-Wise Popular Content Recommendations")

    df["country"] = df["country"].fillna("Unknown")
    all_countries = df["country"].str.split(", ").explode().unique()
    all_countries = sorted([country.strip() for country in all_countries if country != "Unknown"])

    selected_country = st.selectbox("🌐 Select a Country:", all_countries)

    if selected_country:
        country_filtered_df = df[df["country"].str.contains(selected_country, case=False, na=False)]
        popular_content = country_filtered_df[country_filtered_df["rating"] != "Not Rated"]
        rating_priority = {
            "TV-MA": 1, "R": 2, "PG-13": 3, "TV-14": 4, "PG": 5, 
            "G": 6, "NC-17": 7, "TV-Y7": 8, "TV-Y": 9, "TV-G": 10, 
            "TV-PG": 11, "Not Rated": 12
        }
        popular_content["rating_priority"] = popular_content["rating"].map(rating_priority)
        sorted_content = popular_content.sort_values(by=["rating_priority", "release_year"], ascending=[True, False])
        recommendations = sorted_content.head(20)
        st.markdown(f"### 🎬 Top 20 Popular Titles from **{selected_country}**")
        if recommendations.empty:
            st.warning("No popular content found for the selected country. Please choose a different country.")
        else:
            st.write(recommendations[["title", "type", "release_year", "rating", "country"]])
    else:
        st.info("Please select a country to view popular content recommendations.")

    st.markdown("---")  # Section divider

    # 3. Duration Graph
    st.header("📊 3. Duration Trends Analysis")

    df["duration"] = df["duration"].fillna("Unknown")

    def preprocess_duration(row):
        if "min" in row:
            return int(row.replace(" min", ""))
        elif "Season" in row:
            return int(row.replace(" Season", "").replace("s", ""))
        return None

    df["duration_numeric"] = df["duration"].apply(preprocess_duration)

    content_type = st.radio("📽️ Select Content Type:", options=["Movie", "TV Show"])
    available_genres = sorted(set(df["listed_in"].str.split(", ").explode().dropna()))
    selected_genres = st.multiselect("🎞️ Select Genres:", available_genres, default=[])

    filtered_df = df[
        (df["type"] == content_type) &
        (df["listed_in"].apply(lambda genres: any(genre in genres for genre in selected_genres))) &
        (df["duration_numeric"].notnull())
    ]

    duration_analysis = (
        filtered_df.groupby("release_year")["duration_numeric"]
        .mean()
        .reset_index()
        .rename(columns={"duration_numeric": "Average Duration"})
    )

    st.markdown("### 📈 Duration Trends Over Time")
    if not duration_analysis.empty:
        fig, ax = plt.subplots(figsize=(10, 6))
        ax.plot(
            duration_analysis["release_year"],
            duration_analysis["Average Duration"],
            marker="o",
            linestyle="-",
            color="#FF6347",
        )
        ax.set_title(f"Average Duration of {content_type}s Over Time", fontsize=16)
        ax.set_xlabel("Release Year")
        ax.set_ylabel("Average Duration (Minutes/Seasons)")
        st.pyplot(fig)
    else:
        st.warning("No data available for the selected criteria.")

model_application_page()
