import streamlit as st
import matplotlib.pyplot as plt
import seaborn as sns
import pandas as pd
import numpy as np

from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler, LabelEncoder
from sklearn.metrics import classification_report, confusion_matrix


# Function to load data
def load_data(file_path):
    """Load the Netflix dataset from the provided file path."""
    return pd.read_csv(file_path)


# Function to preprocess data
def preprocess_data(df: pd.DataFrame):
    """Preprocess the dataset: handle missing values, clean columns, encode features, and scale numeric data."""
    # Handle missing values in key columns
    df = df.dropna(subset=['duration', 'director', 'release_year', 'listed_in'])

    # Clean 'duration' column
    def clean_duration(value):
        if "min" in str(value):
            return int(value.replace(" min", "").strip())
        return 0  # Handle 'Season' or unrecognized patterns

    df['duration'] = df['duration'].apply(clean_duration)
    df = df[df['duration'] >= 0]  # Remove invalid durations

    # Extract and encode primary genre
    df['primary_genre'] = df['listed_in'].str.split(',').str[0].str.strip()
    genre_counts = df['primary_genre'].value_counts()
    common_genres = genre_counts[genre_counts >= 5].index
    df['primary_genre'] = df['primary_genre'].apply(lambda g: g if g in common_genres else 'Other')

    # Encode genres and directors
    genre_encoder = LabelEncoder()
    df['genre_encoded'] = genre_encoder.fit_transform(df['primary_genre'])

    director_counts = df['director'].value_counts()
    df['director_count'] = df['director'].map(director_counts)

    # Scale numeric features
    duration_scaler = StandardScaler()
    release_year_scaler = StandardScaler()
    df['duration_scaled'] = duration_scaler.fit_transform(df[['duration']])
    df['release_year_scaled'] = release_year_scaler.fit_transform(df[['release_year']])

    return df, genre_encoder, duration_scaler, release_year_scaler, director_counts


# Function to train Logistic Regression
def train_logistic_regression(X_train, y_train):
    """Train a Logistic Regression model on the training dataset."""
    model = LogisticRegression(multi_class='multinomial', solver='lbfgs', max_iter=1000, random_state=42)
    model.fit(X_train, y_train)
    return model


# Evaluate the model
def evaluate_model(y_test, y_pred, encoder):
    """Display classification metrics and a confusion matrix."""
    report = classification_report(y_test, y_pred, target_names=encoder.classes_)
    st.markdown("### Classification Report")
    st.text(report)


# Plot correlation heatmap
def plot_correlation(df: pd.DataFrame):
    """Visualize correlation between numeric features and the target."""
    st.markdown("### Correlation Heatmap")
    corr_cols = ['duration', 'release_year', 'director_count', 'genre_encoded']
    corr = df[corr_cols].corr()
    fig, ax = plt.subplots(figsize=(8, 6))
    sns.heatmap(corr, annot=True, cmap='coolwarm', ax=ax)
    st.pyplot(fig)


# Visualize feature coefficients
def plot_feature_coefficients(model, features):
    """Visualize feature coefficients from the Logistic Regression model."""
    st.markdown("### Feature Coefficients")
    mean_abs_coef = np.mean(np.abs(model.coef_), axis=0)
    coef_series = pd.Series(mean_abs_coef, index=features).sort_values()

    fig, ax = plt.subplots(figsize=(8, 6))
    coef_series.plot(kind='barh', color='skyblue', ax=ax)
    ax.set_title("Average Absolute Feature Coefficients")
    st.pyplot(fig)


def visualize_custom_prediction(df: pd.DataFrame, encoder, custom_genre: str):
    """Highlight the custom predicted genre in a bar chart with a legend."""
    # Generate color palette: red for custom genre, blue for others
    palette = ['red' if genre == custom_genre else 'blue' for genre in encoder.classes_]
    
    # Create the plot
    fig, ax = plt.subplots(figsize=(8, 4))
    sns.countplot(
        y='predicted_genre', 
        data=df, 
        order=encoder.classes_, 
        palette=palette, 
        ax=ax
    )
    ax.set_title("Predicted Genre Distribution (Custom Prediction Highlighted)")
    ax.set_xlabel("Count")
    ax.set_ylabel("Genre")
    
    # Add legend
    handles = [
        plt.Line2D([0], [0], marker='o', color='red', label="Custom Prediction", markersize=10, linestyle=''),
        plt.Line2D([0], [0], marker='o', color='blue', label="Data", markersize=10, linestyle='')
    ]
    ax.legend(handles=handles, loc='upper right', title="Legend")
    
    # Display the plot in Streamlit
    st.pyplot(fig)



# Main application
def main():
    st.title("🎬 Netflix Genre Prediction using Logistic Regression")
    st.markdown(
        """
        <style>
        .header {
            background-color: #34495e;
            color: white;
            padding: 15px;
            border-radius: 10px;
            text-align: center;
            font-size: 20px;
        }
        </style>
        <div class="header">
            Analyze, predict, and visualize genres in the Netflix dataset using Logistic Regression!
        </div>
        """,
        unsafe_allow_html=True,
    )

    # Load data
    file_path = "netflix_titles.csv"
    df = load_data(file_path)
    if st.checkbox("Show Raw Data"):
        st.dataframe(df.head())

    # Preprocess data
    df, genre_encoder, duration_scaler, release_year_scaler, director_counts = preprocess_data(df)

    # Feature and Target Selection
    st.markdown("## 📊 Feature and Target Selection")
    features = ["duration_scaled", "director_count", "release_year_scaled"]
    target_col = "genre_encoded"
    X, y = df[features], df[target_col]

    st.write("### Features:")
    st.dataframe(X.head())
    st.write("### Target (Encoded Genres):")
    st.dataframe(y.head())

    # Train-test split
    st.markdown("## ✂️ Splitting Data")
    test_size = st.slider("Test Set Size (%)", min_value=10, max_value=50, value=20, step=5)
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size / 100, random_state=42)

    st.write(f"Training Set Size: {len(X_train)}")
    st.write(f"Testing Set Size: {len(X_test)}")

    # Train Logistic Regression
    st.markdown("## 🤖 Training Logistic Regression")
    model = train_logistic_regression(X_train, y_train)
    st.success("Logistic Regression Model Trained Successfully!")


    # Correlation heatmap
    plot_correlation(df)

    # Predict on entire dataset
    df["predicted_genre"] = genre_encoder.inverse_transform(model.predict(X))
    st.markdown("### Sample Predictions:")
    st.dataframe(df[["duration", "release_year", "director_count", "primary_genre", "predicted_genre"]].head())

    # Custom prediction
    st.markdown("## 🎥 Make a Custom Prediction")

    # Add a default option for directors
    unique_directors = ["Select a Director"] + sorted(df["director"].unique())
    selected_director = st.selectbox("Select Director", unique_directors)

    # Initialize placeholders for inputs
    custom_genre = None

    if selected_director != "Select a Director":
        # Proceed if a valid director is selected
        director_count = director_counts.get(selected_director, 0)
        
        # Collect other inputs
        duration_choice = st.slider(
            "Duration (minutes)", 
            min_value=int(df["duration"].min()), 
            max_value=int(df["duration"].max())
        )
        release_year_choice = st.slider(
            "Release Year", 
            min_value=int(df["release_year"].min()), 
            max_value=int(df["release_year"].max())
        )

        # Scale inputs and make a prediction
        try:
            duration_scaled_input = duration_scaler.transform([[duration_choice]])[0][0]
            release_year_scaled_input = release_year_scaler.transform([[release_year_choice]])[0][0]
            custom_input = [[duration_scaled_input, director_count, release_year_scaled_input]]
            custom_pred_label = model.predict(custom_input)[0]
            custom_genre = genre_encoder.inverse_transform([custom_pred_label])[0]
        except Exception as e:
            st.error(f"Error during prediction: {e}")
    else:
        # Prompt the user to select a valid director
        st.warning("Please select a Director to make a Prediction.")

    # Display the predicted genre if available
    if custom_genre:
        st.markdown(f"### Predicted Genre: 🎬 **{custom_genre}**")
    else:
        st.markdown("### Predicted Genre: Please provide all inputs to get a prediction.")

    visualize_custom_prediction(df, genre_encoder, custom_genre)


# Run the application
main()
