import streamlit as st
import pandas as pd
from sklearn.preprocessing import MinMaxScaler, StandardScaler
import numpy as np

def preprocessing_page():
    # Title with a styled header
    st.title("📊 Netflix Dataset Preprocessing")
    st.markdown(
        """
        <style>
        .header {
            background-color: #f39c12;
            color: white;
            padding: 15px;
            border-radius: 10px;
            text-align: center;
            font-size: 20px;
            margin-bottom: 20px;
        }
        </style>
        <div class="header">
            Clean, preprocess, and prepare your Netflix dataset for advanced analytics!
        </div>
        """,
        unsafe_allow_html=True,
    )

    # Load Netflix dataset
    file_path = "netflix_titles.csv"  # Ensure this path is correct
    try:
        # Load the dataset and store it in session state
        df = pd.read_csv(file_path)
        st.session_state.df = df
        st.success("🎉 Dataset loaded successfully!")
    except FileNotFoundError:
        # Error handling if the file is not found
        st.error(f"❌ File not found at path: {file_path}. Please check the file path and try again.")
        return

    st.markdown("---")  # Divider

    # 1. Display original dataset
    st.header("🔍 Original Dataset")
    st.write(df.head())  # Display the first few rows

    # 2. Summary Statistics Before Cleaning
    st.header("📈 Summary Statistics (Before Cleaning)")
    st.write(df.describe(include='all'))  # Display all columns' statistics

    # 3. Missing Values Overview
    st.header("🛠️ Missing Values Overview")
    missing_values = df.isnull().sum()  # Calculate missing values per column
    st.write("### Missing Values per Column:")
    st.write(missing_values)

    # 4. Removing All Rows with Any Missing Values
    st.header("🧹 Removing All Missing Values")
    num_rows_before = df.shape[0]
    num_missing = missing_values.sum()
    st.write(f"**Total Rows Before Cleaning:** {num_rows_before}")
    st.write(f"**Total Missing Values:** {num_missing}")

    if num_missing > 0:
        # Drop rows with missing values
        df = df.dropna()
        num_rows_after = df.shape[0]
        rows_removed = num_rows_before - num_rows_after
        st.success(f"✅ Missing values removed! Total rows after cleaning: {num_rows_after}")
        st.info(f"🧾 Rows Removed: {rows_removed}")
    else:
        # Notify the user if no missing values are found
        st.success("✅ No missing values found. Dataset is already clean.")

    st.markdown("---")  # Divider

    # 5. Summary Statistics After Cleaning
    st.header("📊 Summary Statistics (After Cleaning)")
    st.write(df.describe(include='all'))

    # 6. Scaling: Min-Max and Standardization
    st.header("⚖️ Scaling: Min-Max Scaling and Standardization")
    numeric_cols = df.select_dtypes(include=['float64', 'int64']).columns.tolist()  # Identify numeric columns

    if numeric_cols:
        # Min-Max Scaling
        st.subheader("🔹 Min-Max Scaled Data")
        minmax_scaler = MinMaxScaler()
        df_minmax_scaled = pd.DataFrame(minmax_scaler.fit_transform(df[numeric_cols]), columns=numeric_cols)
        st.write(df_minmax_scaled.head())

        # Standardization
        st.subheader("🔹 Standardized Data")
        standard_scaler = StandardScaler()
        df_standardized = pd.DataFrame(standard_scaler.fit_transform(df[numeric_cols]), columns=numeric_cols)
        st.write(df_standardized.head())
    else:
        # Notify the user if no numeric columns are available for scaling
        st.warning("⚠️ No numeric columns available for scaling in the dataset.")

    st.markdown("---")  # Divider

    # 7. Outlier Detection and Removal
    st.header("🚨 Outlier Detection and Removal")
    st.write("Identifying outliers using the Interquartile Range (IQR) method.")

    # Function to remove outliers using IQR
    def remove_outliers_iqr(data, column):
        Q1 = data[column].quantile(0.25)
        Q3 = data[column].quantile(0.75)
        IQR = Q3 - Q1
        lower_bound = Q1 - 1.5 * IQR
        upper_bound = Q3 + 1.5 * IQR
        return data[(data[column] >= lower_bound) & (data[column] <= upper_bound)]

    # Apply outlier removal to each numeric column
    for col in numeric_cols:
        initial_count = df.shape[0]
        df = remove_outliers_iqr(df, col)
        final_count = df.shape[0]
        rows_removed = initial_count - final_count
        st.write(f"**Outliers Removed from '{col}':** {rows_removed} rows")

    st.markdown("---")  # Divider

    # 8. Dataset After Removing Outliers
    st.header("📜 Dataset After Removing Outliers")
    st.write(df.describe(include='all'))  # Display updated statistics

    # Update session state with the final cleaned dataframe
    st.session_state.df = df

    # 9. Mark Preprocessing as Complete
    st.success("🎉 Preprocessing Complete! Your dataset is clean and ready for further analysis.")

# Run the preprocessing page function
preprocessing_page()
