import streamlit as st
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

def visualization_page():
    st.title("Netflix Dataset Visualization")

    # Load updated dataset
    file_path = "netflix_titles.csv"  # Path to Netflix dataset (with or without fake data)
    df = pd.read_csv(file_path)

    # Check if fake data was included (assumes a column or indicator exists for fake data)
    if "is_fake" not in df.columns:
        df["is_fake"] = 0  # Default to original data

    # Dataset Overview
    st.markdown("## Dataset Overview")
    st.write(df.head())
    st.markdown("### Data Summary")
    st.write(df.describe(include="all"))

    # Original vs. Fake Data
    st.markdown("## Original vs. Fake Data Distribution")
    original_count = df[df["is_fake"] == 0].shape[0]
    fake_count = df[df["is_fake"] == 1].shape[0]
    st.write(f"Original Data Count: {original_count}")
    st.write(f"Fake Data Count: {fake_count}")

    # Countplot of Original vs. Fake
    st.markdown("### Original vs. Fake Data Count")
    fig, ax = plt.subplots()
    sns.countplot(data=df, x="is_fake", ax=ax)
    ax.set_xticklabels(["Original Data", "Fake Data"])
    ax.set_title("Original vs. Fake Data Count")
    st.pyplot(fig)

    # Visualization: Type Distribution
    st.markdown("## Content Type Distribution (Movies vs. TV Shows)")
    fig, ax = plt.subplots()
    sns.countplot(data=df, x="type", hue="is_fake", ax=ax)
    ax.set_title("Content Type Distribution (Original vs. Fake Data)")
    st.pyplot(fig)

    # Visualization: Rating Distribution
    st.markdown("## Rating Distribution")
    fig, ax = plt.subplots(figsize=(12, 6))
    sns.countplot(data=df, x="rating", hue="is_fake", ax=ax, order=df["rating"].value_counts().index)
    ax.set_title("Rating Distribution (Original vs. Fake Data)")
    ax.set_xticklabels(ax.get_xticklabels(), rotation=45)
    st.pyplot(fig)

    # Release Year Distribution
    st.markdown("## Release Year Distribution")
    fig, ax = plt.subplots(figsize=(12, 6))
    sns.histplot(data=df, x="release_year", hue="is_fake", kde=True, bins=30, ax=ax)
    ax.set_title("Release Year Distribution (Original vs. Fake Data)")
    st.pyplot(fig)

    # Visualization: Title Age
    if "release_year" in df.columns:
        df["title_age"] = 2024 - df["release_year"]
        st.markdown("## Title Age Distribution")
        fig, ax = plt.subplots(figsize=(12, 6))
        sns.histplot(data=df, x="title_age", hue="is_fake", kde=True, bins=30, ax=ax)
        ax.set_title("Title Age Distribution (Original vs. Fake Data)")
        st.pyplot(fig)

    # Country Distribution
    st.markdown("## Country Distribution")
    top_countries = df["country"].value_counts().head(10).index
    filtered_df = df[df["country"].isin(top_countries)]
    fig, ax = plt.subplots(figsize=(12, 6))
    sns.countplot(data=filtered_df, y="country", hue="is_fake", ax=ax)
    ax.set_title("Top 10 Countries with Most Titles (Original vs. Fake Data)")
    st.pyplot(fig)

    # Correlation Heatmap
    st.markdown("## Correlation Heatmap")
    numeric_cols = df.select_dtypes(include=["float64", "int64"]).columns
    fig, ax = plt.subplots(figsize=(10, 8))
    sns.heatmap(df[numeric_cols].corr(), annot=True, cmap="coolwarm", ax=ax)
    ax.set_title("Correlation Heatmap of Numeric Features")
    st.pyplot(fig)

    st.markdown("### Dataset Visualization Complete")

    # Save the visualization data to session state for continuity
    st.session_state.visualization_data = df

visualization_page()